@@ -1,7 +1,7 @@
 /*
  *  callbacks.c - functions to handle GUI events.
  *	part of galculator
- *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2009 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,18 +37,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
 
-#define SELECT_RESULT_FONT _("Select result font")
-#define SELECT_STACK_FONT _("Select stack font")
-#define SELECT_MODULE_FONT _("Select module font")
-#define SELECT_ACT_MOD_COLOR _("Select active module color")
-#define SELECT_INACT_MOD_COLOR _("Select inactive module color")
-#define SELECT_RESULT_FONT_COLOR _("Select result font color")
-#define SELECT_STACK_COLOR _("Select stack color")
-#define SELECT_BKG_COLOR _("Select background color")
-#define SELECT_BUTTON_FONT _("Select button font")	
-
-static GtkWidget		*font_dialog, *color_dialog;
-
 /* File */
 
 void
@@ -58,28 +46,31 @@ on_quit_activate                      (GtkMenuItem     *menuitem,
 	char 	**stack;
 	
 	/* remember some things */
-	if (prefs.mode == SCIENTIFIC_MODE) {
+	if (prefs.mode != BASIC_MODE) {
 		/* save number and angle mode only in scientific mode. */
 		prefs.def_number = current_status.number;
 		prefs.def_angle = current_status.angle;
 	}
-	if (prefs.rem_valuex) g_free (prefs.rem_valuex);
-	prefs.rem_valuex = display_result_get();
-	if (current_status.notation == CS_RPN) {
-		if (prefs.rem_valuey) g_free (prefs.rem_valuey);
-		if (prefs.rem_valuez) g_free (prefs.rem_valuez);
-		if (prefs.rem_valuet) g_free (prefs.rem_valuet);
-		/* we only save the visible stack */
-		stack = display_stack_get_yzt();
-		prefs.rem_valuey = g_strdup(stack[0]);
-		g_free (stack[0]);
-		prefs.rem_valuez = g_strdup(stack[1]);
-		g_free (stack[1]);
-		prefs.rem_valuet = g_strdup(stack[2]);
-		g_free (stack[2]);
-		g_free (stack);
+	/* if we have the classic view display we remember the display values */
+	if (prefs.mode != PAPER_MODE) {
+		if (prefs.rem_valuex) g_free (prefs.rem_valuex);
+		prefs.rem_valuex = display_result_get();
+		if (current_status.notation == CS_RPN) {
+			if (prefs.rem_valuey) g_free (prefs.rem_valuey);
+			if (prefs.rem_valuez) g_free (prefs.rem_valuez);
+			if (prefs.rem_valuet) g_free (prefs.rem_valuet);
+			/* we only save the visible stack */
+			stack = display_stack_get_yzt();
+			prefs.rem_valuey = g_strdup(stack[0]);
+			g_free (stack[0]);
+			prefs.rem_valuez = g_strdup(stack[1]);
+			g_free (stack[1]);
+			prefs.rem_valuet = g_strdup(stack[2]);
+			g_free (stack[2]);
+			g_free (stack);
+		}
+		prefs.def_notation = current_status.notation;
 	}
-	prefs.def_notation = current_status.notation;
 	gtk_main_quit();
 }
 
@@ -100,7 +91,7 @@ on_about_activate                     (GtkMenuItem     *menuitem,
 	gtk_label_set_justify (about_label, GTK_JUSTIFY_CENTER);
 	about_text = g_strdup_printf (_("<span size=\"x-large\" weight=\"bold\">%s v%s</span>\n\
 <span size=\"large\">a GTK 2 based scientific calculator</span>\n\n\
-(c) 2002-2004 by Simon Floery (chimaira@users.sf.net)"), PROG_NAME, VERSION);
+(c) 2002-2009 by Simon Floery (chimaira@users.sf.net)"), PROG_NAME, VERSION);
 	gtk_label_set_markup (about_label, \
 		about_text);
 	gtk_widget_show (about_dialog);
@@ -128,7 +119,7 @@ on_number_button_clicked               (GtkToggleButton  *button,
 		ui_formula_entry_insert (gtk_button_get_label ((GtkButton *)button));
 	} else {
 		rpn_stack_lift();
-		display_result_add_digit (*(gtk_button_get_label ((GtkButton *)button)));
+		display_result_add_digit (*(gtk_button_get_label ((GtkButton *)button)), current_status.number);
 	}
 	return;
 }
@@ -149,7 +140,7 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 	button_activation (button);
 	
 	if (current_status.notation == CS_FORMULA) {
-		if (strcmp (gtk_widget_get_name ((GtkWidget *) button), "button_enter") == 0)
+		if (strcmp (gtk_widget_get_name((GtkWidget *) button), "button_enter") == 0)
 			ui_formula_entry_activate();
 		/* as long as we don't support string operation ids, we take
 		 * operation char. take this later on:
@@ -157,17 +148,17 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		 *	g_object_get_data (G_OBJECT (button), "display_string"));
 		 */
 		else {
-			operation_string = g_strdup_printf ("%c", (int) g_object_get_data (
-				G_OBJECT (button), "operation"));
+			operation_string = g_strdup_printf("%c", GPOINTER_TO_INT(g_object_get_data (
+				G_OBJECT (button), "operation")));
 			ui_formula_entry_insert (operation_string);
 			g_free(operation_string);
 		}
 		return;
 	}
 	
-	current_token.operation = (int) g_object_get_data (G_OBJECT (button), "operation");
+	current_token.operation = GPOINTER_TO_INT(g_object_get_data(G_OBJECT (button), "operation"));
 	/* current number, get it from the display! */
-	current_token.number = display_result_get_double ();
+	current_token.number = display_result_get_double (current_status.number);
 	current_token.func = NULL;
 	/* do inverse left shift is a right shift */
 	if ((current_token.operation == '<') && \
@@ -192,7 +183,7 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		if (((current_token.operation == '(') || current_status.allow_arith_op) && \
 			((current_token.operation != ')') || (display_module_bracket_label_update (GET) > 0))) {
 			return_value = alg_add_token (&main_alg, current_token);
-			display_result_set_double (return_value);
+			display_result_set_double (return_value, current_status.number);
 			display_module_arith_label_update (current_token.operation);
 			
 			/* setting of allow_arith_op. the missing breaks are wanted */
@@ -214,16 +205,16 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		case '=':
 			rpn_stack_push (current_token.number);
 			stack = rpn_stack_get (RPN_FINITE_STACK);
-			display_stack_set_yzt_double (stack);
+			display_stack_set_yzt_double (stack, current_status.number);
 			free (stack);
 			/* ENT is a stack lift disabling button */
 			current_status.rpn_stack_lift_enabled = FALSE;
 			/* display line isn't cleared! */
 			break;
 		default:
-			display_result_set_double (rpn_stack_operation (current_token));
+			display_result_set_double (rpn_stack_operation (current_token), current_status.number);
 			stack = rpn_stack_get (RPN_FINITE_STACK);
-			display_stack_set_yzt_double (stack);
+			display_stack_set_yzt_double (stack, current_status.number);
 			free (stack);
 			/* all other operations are stack lift enabling */
 			current_status.rpn_stack_lift_enabled = TRUE;
@@ -255,9 +246,9 @@ on_function_button_clicked             (GtkToggleButton	*button,
 		return;
 	}
 	memcpy (func, g_object_get_data (G_OBJECT (button), "func"), sizeof (func));
-	if (!func) error_message ("This button has no function associated with");
+	if (!*func) error_message ("This button has no function associated with");
 	display_result_set_double (
-		func[current_status.fmod](display_result_get_double()));
+		func[current_status.fmod](display_result_get_double(current_status.number)), current_status.number);
 	current_status.calc_entry_start_new = TRUE;	
 	if (current_status.notation == CS_RPN) 
 		current_status.rpn_stack_lift_enabled = TRUE;
@@ -311,8 +302,7 @@ on_dec_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("dec\n");*/
-	display_change_option (CS_DEC, DISPLAY_OPT_NUMBER);
+	change_option (CS_DEC, DISPLAY_OPT_NUMBER);
 }
 
 
@@ -321,8 +311,7 @@ on_hex_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("hex\n");*/
-	display_change_option (CS_HEX, DISPLAY_OPT_NUMBER);
+	change_option (CS_HEX, DISPLAY_OPT_NUMBER);
 }
 
 
@@ -331,8 +320,7 @@ on_oct_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("oct\n");*/
-	display_change_option (CS_OCT, DISPLAY_OPT_NUMBER);
+	change_option (CS_OCT, DISPLAY_OPT_NUMBER);
 }
 
 
@@ -341,8 +329,7 @@ on_bin_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("bin\n");*/
-	display_change_option (CS_BIN, DISPLAY_OPT_NUMBER);
+	change_option (CS_BIN, DISPLAY_OPT_NUMBER);
 }
 
 
@@ -351,8 +338,7 @@ on_deg_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("deg\n");*/
-	display_change_option (CS_DEG, DISPLAY_OPT_ANGLE);
+	change_option (CS_DEG, DISPLAY_OPT_ANGLE);
 }
 
 
@@ -361,8 +347,7 @@ on_rad_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("rad\n");*/
-	display_change_option (CS_RAD, DISPLAY_OPT_ANGLE);
+	change_option (CS_RAD, DISPLAY_OPT_ANGLE);
 }
 
 
@@ -371,8 +356,7 @@ on_grad_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	/*printf ("grad\n");*/
-	display_change_option (CS_GRAD, DISPLAY_OPT_ANGLE);
+	change_option (CS_GRAD, DISPLAY_OPT_ANGLE);
 }
 
 void
@@ -380,8 +364,8 @@ on_ordinary_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	display_change_option (CS_PAN, DISPLAY_OPT_NOTATION);
-	set_widget_visibility (main_window_xml, "formula_entry_hbox", FALSE);
+	change_option (CS_PAN, DISPLAY_OPT_NOTATION);
+	set_widget_visibility (view_xml, "formula_entry_hbox", FALSE);
 	rpn_free();
 	all_clear();
 	ui_button_set_pan();
@@ -396,8 +380,9 @@ on_rpn_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	display_change_option (CS_RPN, DISPLAY_OPT_NOTATION);
-	set_widget_visibility (main_window_xml, "formula_entry_hbox", FALSE);
+	change_option (CS_RPN, DISPLAY_OPT_NOTATION);
+	
+	set_widget_visibility (view_xml, "formula_entry_hbox", FALSE);
 	alg_free(main_alg);
 	all_clear();
 	ui_button_set_rpn();
@@ -412,17 +397,18 @@ on_form_activate 			(GtkMenuItem     *menuitem,
 					gpointer         user_data)
 {
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	display_change_option (CS_FORMULA, DISPLAY_OPT_NOTATION);
+	change_option (CS_FORMULA, DISPLAY_OPT_NOTATION);
 	all_clear();
 	ui_button_set_pan();
 	update_dispctrl();
-	set_widget_visibility (main_window_xml, "formula_entry_hbox", TRUE);	
+	set_widget_visibility (view_xml, "formula_entry_hbox", TRUE);	
 }
 
 void
 on_display_control_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
+	if (prefs.mode == PAPER_MODE) return;
 	prefs.vis_dispctrl = 
 		gtk_check_menu_item_get_active((GtkCheckMenuItem *) menuitem);
 	set_widget_visibility (dispctrl_xml, "table_dispctrl", 
@@ -434,6 +420,7 @@ on_logical_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
 	if (prefs.mode == BASIC_MODE) return;
+	if (prefs.mode == PAPER_MODE) return;
 
 	prefs.vis_logic = 
 		gtk_check_menu_item_get_active((GtkCheckMenuItem *) menuitem);
@@ -446,6 +433,7 @@ on_functions_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
 	if (prefs.mode == BASIC_MODE) return;
+	if (prefs.mode == PAPER_MODE) return;
 	prefs.vis_funcs = 
 		gtk_check_menu_item_get_active((GtkCheckMenuItem *) menuitem);
 	set_widget_visibility (button_box_xml, "table_func_buttons",
@@ -457,6 +445,7 @@ on_standard_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
 	if (prefs.mode == BASIC_MODE) return;
+	if (prefs.mode == PAPER_MODE) return;
 	prefs.vis_standard = 
 		gtk_check_menu_item_get_active((GtkCheckMenuItem *) menuitem);
 	set_widget_visibility (button_box_xml, "table_standard_buttons",
@@ -477,12 +466,14 @@ on_basic_mode_activate (GtkMenuItem     *menuitem,
 	}
 	prefs.mode = BASIC_MODE;
 	
+	ui_paper_view_destroy ();
+	ui_classic_view_create ();
 	ui_main_window_buttons_destroy ();
 	ui_main_window_buttons_create (prefs.mode);
 	update_dispctrl();
 	
 	display_update_modules();
-
+	
 	/* In basic mode:
 	 *	- number base is always decimal.
 	 *	- ignore angle, as there are no angle operations in basic mode.
@@ -490,7 +481,7 @@ on_basic_mode_activate (GtkMenuItem     *menuitem,
 	 */	
 	display_module_number_activate (CS_DEC);
 	display_module_notation_activate (current_status.notation);
-	
+
 	menu_item = glade_xml_get_widget (main_window_xml, "display_control");
 	if (((GtkCheckMenuItem *) menu_item)->active == prefs.vis_dispctrl)
 			gtk_menu_item_activate ((GtkMenuItem *) menu_item);
@@ -507,6 +498,12 @@ on_basic_mode_activate (GtkMenuItem     *menuitem,
 	gtk_widget_set_sensitive (menu_item, FALSE);
 	menu_item = glade_xml_get_widget (main_window_xml, "angle_units");
 	gtk_widget_set_sensitive (menu_item, FALSE);
+	menu_item = glade_xml_get_widget (main_window_xml, "buttons1");
+	gtk_widget_set_sensitive (menu_item, TRUE);
+	menu_item = glade_xml_get_widget (main_window_xml, "notation");
+	gtk_widget_set_sensitive (menu_item, TRUE);
+	
+	set_window_size_minimal();
 }
 
 void
@@ -518,9 +515,11 @@ on_scientific_mode_activate (GtkMenuItem *menuitem,
 	if (((GtkCheckMenuItem *) menuitem)->active == FALSE) return;
 	prefs.mode = SCIENTIFIC_MODE;
 
+	ui_paper_view_destroy ();
+	ui_classic_view_create ();
 	ui_main_window_buttons_destroy ();
 	ui_main_window_buttons_create (prefs.mode);
-
+	
 	display_update_modules();
 	display_module_number_activate (prefs.def_number);
 	display_module_angle_activate (prefs.def_angle);
@@ -561,12 +560,46 @@ on_scientific_mode_activate (GtkMenuItem *menuitem,
 	gtk_widget_set_sensitive (menu_item, TRUE);
 	menu_item = glade_xml_get_widget (main_window_xml, "angle_units");
 	gtk_widget_set_sensitive (menu_item, TRUE);
+	menu_item = glade_xml_get_widget (main_window_xml, "buttons1");
+	gtk_widget_set_sensitive (menu_item, TRUE);
+	menu_item = glade_xml_get_widget (main_window_xml, "notation");
+	gtk_widget_set_sensitive (menu_item, TRUE);
+	
+	set_window_size_minimal();
+}	
+
+void
+on_paper_mode_activate (GtkMenuItem *menuitem,
+				gpointer user_data)
+{
+	GtkWidget	*menu_item;
+	
+	if (((GtkCheckMenuItem *) menuitem)->active == FALSE) return;
+	prefs.mode = PAPER_MODE;
+
+	ui_classic_view_destroy();
+	ui_paper_view_create ();
+	
+	/* this is to get the radio menu items right. display* naming is 
+	 * misleading though ...
+	 */
+	display_module_number_activate (prefs.def_number);
+	display_module_angle_activate (prefs.def_angle);
+	
+	/* update menus */
+	menu_item = glade_xml_get_widget (main_window_xml, "buttons1");
+	gtk_widget_set_sensitive (menu_item, FALSE);
+	menu_item = glade_xml_get_widget (main_window_xml, "notation");
+	gtk_widget_set_sensitive (menu_item, FALSE);
+	
+	set_window_size_minimal();
 }	
 
 void
 on_cut_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
+	if (prefs.mode == PAPER_MODE) return;
 	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), 
 		display_result_get(), -1);
 	clear ();
@@ -579,12 +612,13 @@ on_paste_activate (GtkMenuItem     *menuitem,
 	GtkWidget	*formula_entry;
 	char		*cp_text;
 	
+	if (prefs.mode == PAPER_MODE) return;
 	cp_text = gtk_clipboard_wait_for_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
 	if (cp_text) {
 		if ((formula_entry = formula_entry_is_active_no_toplevel_check ()) != NULL) {
 			gtk_editable_paste_clipboard((GtkEditable *)formula_entry);
 		}
-		else display_result_feed (cp_text);
+		else display_result_feed (cp_text, current_status.number);
 		g_free (cp_text);
 	}
 }
@@ -593,6 +627,7 @@ void
 on_copy_activate (GtkMenuItem     *menuitem,
 			gpointer         user_data)
 {
+	if (prefs.mode == PAPER_MODE) return;
 	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), 
 		display_result_get(), -1);
 }
@@ -657,179 +692,107 @@ on_preferences1_activate               (GtkMenuItem     *menuitem,
 }
 
 void
-on_prefs_result_font_clicked           (GtkButton       *button,
-                                        gpointer         user_data)
+on_prefs_result_font_set(GtkFontButton *button, gpointer user_data)
 {
-	font_dialog = ui_font_dialog_create (SELECT_RESULT_FONT, button);
+	const char	*font_name;
+	
+	font_name = gtk_font_button_get_font_name(button);
+	if (prefs.result_font != NULL) g_free (prefs.result_font);
+	prefs.result_font = g_strdup(font_name);
+	display_update_tags();
 }
 
 void
-on_prefs_result_color_clicked          (GtkButton       *button,
-                                        gpointer         user_data)
+on_prefs_result_color_set(GtkColorButton *button, gpointer user_data)
 {
-	color_dialog = ui_color_dialog_create (SELECT_RESULT_FONT_COLOR, button);
-}
-
-void
-on_prefs_stack_font_clicked           (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	font_dialog = ui_font_dialog_create (SELECT_STACK_FONT, button);
-}
-
-void
-on_prefs_stack_color_clicked          (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	color_dialog = ui_color_dialog_create (SELECT_STACK_COLOR, button);
-}
-
-void
-on_prefs_mod_font_clicked              (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	font_dialog = ui_font_dialog_create (SELECT_MODULE_FONT, button);
-}
-
-
-void
-on_prefs_act_mod_color_clicked         (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	color_dialog = ui_color_dialog_create (SELECT_ACT_MOD_COLOR, button);
-}
-
-
-void
-on_prefs_inact_mod_color_clicked       (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	color_dialog = ui_color_dialog_create (SELECT_INACT_MOD_COLOR, button);
-}
-
-
-void
-on_prefs_bkg_color_clicked             (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	color_dialog = ui_color_dialog_create (SELECT_BKG_COLOR, button);
-}
-
-
-void
-on_prefs_button_font_clicked           (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	font_dialog = ui_font_dialog_create (SELECT_BUTTON_FONT, button);
-}
-
-void
-on_color_ok_button_clicked             (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	const char 	*title;
-	GtkWidget	*da=NULL;
 	GdkColor	color;
-
-	title = gtk_window_get_title ((GtkWindow *) color_dialog);
-	gtk_color_selection_get_current_color ((GtkColorSelection *)(((GtkColorSelectionDialog *) color_dialog)->colorsel), \
-		&color);
 	
-	if (strcmp (title, SELECT_BKG_COLOR) == 0)
-	{
-		da = glade_xml_get_widget (prefs_xml, "prefs_bkg_color");
-		if (prefs.bkg_color != NULL) g_free (prefs.bkg_color);
-		prefs.bkg_color = gdk_color_to_string(color);
-		display_set_bkg_color (prefs.bkg_color);
-	}
-	else if (strcmp (title, SELECT_RESULT_FONT_COLOR) == 0)
-	{
-		da = glade_xml_get_widget (prefs_xml, "prefs_result_color");
-		if (prefs.result_color != NULL) g_free (prefs.result_color);
-		prefs.result_color = gdk_color_to_string(color);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_STACK_COLOR) == 0)
-	{
-		da = glade_xml_get_widget (prefs_xml, "prefs_stack_color");
-		if (prefs.stack_color != NULL) g_free (prefs.stack_color);
-		prefs.stack_color = gdk_color_to_string(color);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_ACT_MOD_COLOR) == 0)
-	{
-		da = glade_xml_get_widget (prefs_xml, "prefs_act_mod_color");
-		if (prefs.act_mod_color != NULL) g_free (prefs.act_mod_color);
-		prefs.act_mod_color = gdk_color_to_string(color);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_INACT_MOD_COLOR) == 0)
-	{
-		da = glade_xml_get_widget (prefs_xml, "prefs_inact_mod_color");
-		if (prefs.inact_mod_color != NULL) g_free (prefs.inact_mod_color);
-		prefs.inact_mod_color = gdk_color_to_string(color);
-		display_update_tags();
-	}
-	else fprintf (stderr, "[%s] Color Dialog (%s) not found. %s\n", PACKAGE, 
-		title, BUG_REPORT);
-
-	gtk_widget_really_modify_fg (da, color);
-	
-	gtk_widget_destroy (color_dialog);
+	gtk_color_button_get_color (button, &color);
+	if (prefs.result_color != NULL) g_free (prefs.result_color);
+	prefs.result_color = convert_gdk_color_to_string(color);
+	display_update_tags();
 }
 
-
 void
-on_font_ok_button_clicked              (GtkButton       *button,
-                                        gpointer         user_data)
+on_prefs_stack_font_set(GtkFontButton *button, gpointer user_data)
 {
-	GtkButton		*font_button=NULL;
-	const char 		*title;
-	char			*font_name, *button_font;
-
-	title = gtk_window_get_title ((GtkWindow *) font_dialog);
-	font_name = gtk_font_selection_dialog_get_font_name ((GtkFontSelectionDialog *)font_dialog);
+	const char	*font_name;
 	
-	if (strcmp (title, SELECT_RESULT_FONT) == 0)
-	{
-		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_result_font");
-		if (prefs.result_font != NULL) g_free (prefs.result_font);
-		prefs.result_font = g_strdup(font_name);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_STACK_FONT) == 0)
-	{
-		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_stack_font");
-		if (prefs.stack_font != NULL) g_free (prefs.stack_font);
-		prefs.stack_font = g_strdup(font_name);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_MODULE_FONT) == 0)
-	{
-		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_mod_font");
-		if (prefs.mod_font != NULL) g_free (prefs.mod_font);
-		prefs.mod_font = g_strdup(font_name);
-		display_update_tags();
-	}
-	else if (strcmp (title, SELECT_BUTTON_FONT) == 0)
-	{
-		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_button_font");
-		if (prefs.button_font != NULL) g_free (prefs.button_font);
-		prefs.button_font = g_strdup(font_name);
-		if (prefs.custom_button_font == TRUE) button_font = g_strdup (prefs.button_font);
-		else button_font = g_strdup ("");
-		set_all_buttons_font (button_font);	
-		g_free (button_font);
-	}
-	else fprintf (stderr, "[%s] Font Dialog (%s) not found. %s\n", PACKAGE, 
-		title, BUG_REPORT);
-	
-	gtk_widget_destroy (font_dialog);
-	
-	gtk_button_set_label (font_button, font_name);
+	font_name = gtk_font_button_get_font_name(button);
+	if (prefs.stack_font != NULL) g_free (prefs.stack_font);
+	prefs.stack_font = g_strdup(font_name);
+	display_update_tags();
 }
 
+void
+on_prefs_stack_color_set(GtkColorButton *button, gpointer user_data)
+{
+	GdkColor	color;
+	
+	gtk_color_button_get_color (button, &color);
+	if (prefs.stack_color != NULL) g_free (prefs.stack_color);
+	prefs.stack_color = convert_gdk_color_to_string(color);
+	display_update_tags();
+}
 
+void
+on_prefs_mod_font_set(GtkFontButton *button, gpointer user_data)
+{
+	const char	*font_name;
+	
+	font_name = gtk_font_button_get_font_name(button);
+	if (prefs.mod_font != NULL) g_free (prefs.mod_font);
+	prefs.mod_font = g_strdup(font_name);
+	display_update_tags();
+}
+
+void
+on_prefs_act_mod_color_set(GtkColorButton *button, gpointer user_data)
+{
+	GdkColor	color;
+	
+	gtk_color_button_get_color (button, &color);
+	if (prefs.act_mod_color != NULL) g_free (prefs.act_mod_color);
+	prefs.act_mod_color = convert_gdk_color_to_string(color);
+	display_update_tags();
+}
+
+void
+on_prefs_inact_mod_color_set(GtkColorButton *button, gpointer user_data)
+{
+	GdkColor	color;
+	
+	gtk_color_button_get_color (button, &color);
+	if (prefs.inact_mod_color != NULL) g_free (prefs.inact_mod_color);
+	prefs.inact_mod_color = convert_gdk_color_to_string(color);
+	display_update_tags();
+}
+
+void
+on_prefs_bkg_color_set(GtkColorButton *button, gpointer user_data)
+{
+	GdkColor	color;
+	
+	gtk_color_button_get_color (button, &color);
+	if (prefs.bkg_color != NULL) g_free (prefs.bkg_color);
+	prefs.bkg_color = convert_gdk_color_to_string(color);
+	display_set_bkg_color (prefs.bkg_color);
+}
+
+void
+on_prefs_button_font_set(GtkFontButton *button, gpointer user_data)
+{
+	const char	*font_name;
+	char		*button_font;
+	
+	font_name = gtk_font_button_get_font_name(button);
+	if (prefs.button_font != NULL) g_free (prefs.button_font);
+	prefs.button_font = g_strdup(font_name);
+	if (prefs.custom_button_font == TRUE) button_font = g_strdup (prefs.button_font);
+	else button_font = g_strdup ("");
+	set_all_buttons_font (button_font);	
+	g_free (button_font);
+}
 
 void
 on_prefs_close_clicked                (GtkButton       *button,
@@ -838,23 +801,6 @@ on_prefs_close_clicked                (GtkButton       *button,
 	gtk_widget_destroy (gtk_widget_get_toplevel((GtkWidget *)button));
 }
 
-
-void
-on_color_cancel_button_clicked         (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	gtk_widget_destroy (color_dialog);
-}
-
-
-void
-on_font_cancel_button_clicked          (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	gtk_widget_destroy (font_dialog);
-}
-
-
 void
 on_show_menubar1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -947,16 +893,23 @@ void on_prefs_button_width_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
 {
-	prefs.button_width = (int) gtk_spin_button_get_value (spinbutton);
-	set_all_buttons_size (prefs.button_width, prefs.button_height);
+	int new_button_width = (int) gtk_spin_button_get_value (spinbutton);
+	
+	if (new_button_width != prefs.button_width) {
+		prefs.button_width = new_button_width;
+		set_all_buttons_size (prefs.button_width, prefs.button_height);
+	}
 }
 
 void on_prefs_button_height_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
 {	
-	prefs.button_height = (int) gtk_spin_button_get_value (spinbutton);
-	set_all_buttons_size (prefs.button_width, prefs.button_height);
+	int new_button_height = (int) gtk_spin_button_get_value (spinbutton);
+	if (new_button_height != prefs.button_height) {
+		prefs.button_height = new_button_height;
+		set_all_buttons_size (prefs.button_width, prefs.button_height);
+	}
 }
 
 /*
@@ -968,12 +921,12 @@ void user_functions_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	int 			index;
 	s_flex_parser_result	result;
 	
-	index = (int) user_data;
+	index = GPOINTER_TO_INT(user_data);
 	result = compute_user_function (
 		user_function[index].expression, user_function[index].variable,
 		display_result_get());
 	if (!result.error) {
-		display_result_set_double (result.value);
+		display_result_set_double (result.value, current_status.number);
 		current_status.calc_entry_start_new = TRUE;	
 		if (current_status.notation == CS_RPN) 
 			current_status.rpn_stack_lift_enabled = TRUE;
@@ -1007,7 +960,7 @@ void constants_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	current_status.rpn_stack_lift_enabled = TRUE;
 	rpn_stack_lift();
 	const_value = user_data;
-	display_result_set (const_value);
+	display_result_set (const_value, TRUE, string2double(const_value, current_status.number));
 	current_status.rpn_stack_lift_enabled = TRUE;
 	current_status.calc_entry_start_new = TRUE;
 }
@@ -1041,7 +994,7 @@ void ms_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 		memory.data = (double *) realloc (memory.data, (index + 1) * sizeof(double));
 		memory.len++;
 	}
-	memory.data[index] = display_result_get_double();
+	memory.data[index] = display_result_get_double(current_status.number);
 	
 	/* at startup, mr and m+ button are disabled as there is nothing
 	 * to show. now, as there is sth, enable them. see also 
@@ -1055,9 +1008,7 @@ void ms_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	current_status.calc_entry_start_new = TRUE;
 }
 
-void
-on_ms_button_clicked (GtkToggleButton       *button,
-			gpointer user_data)
+void on_ms_button_clicked (GtkToggleButton *button, gpointer user_data)
 {
 	GtkWidget	*menu;
 	
@@ -1076,14 +1027,12 @@ void mr_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	current_status.rpn_stack_lift_enabled = TRUE;
 	rpn_stack_lift();
 	index = GPOINTER_TO_INT(user_data);
-	display_result_set_double(memory.data[index]);
+	display_result_set_double(memory.data[index], current_status.number);
 	current_status.rpn_stack_lift_enabled = TRUE;
 	current_status.calc_entry_start_new = TRUE;
 }
 
-void
-on_mr_button_clicked             (GtkToggleButton       *button,
-				gpointer         user_data)
+void on_mr_button_clicked (GtkToggleButton *button, gpointer user_data)
 {
 	GtkWidget	*menu;;
 	
@@ -1100,12 +1049,10 @@ void mplus_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	int		index;
 	
 	index = GPOINTER_TO_INT(user_data);
-	memory.data[index] += display_result_get_double();
+	memory.data[index] += display_result_get_double(current_status.number);
 }
 
-void
-on_mplus_button_clicked             (GtkToggleButton       *button,
-				gpointer         user_data)
+void on_mplus_button_clicked (GtkToggleButton *button, gpointer user_data)
 {
 	GtkWidget	*menu;
 	
@@ -1154,8 +1101,8 @@ void mx_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	
 	index = GPOINTER_TO_INT(user_data);
 	temp = memory.data[index];
-	memory.data[index] = display_result_get_double();
-	display_result_set_double (temp);
+	memory.data[index] = display_result_get_double(current_status.number);
+	display_result_set_double (temp, current_status.number);
 }
 
 void
@@ -1576,11 +1523,19 @@ void on_togglebutton_released (GtkToggleButton *togglebutton,
 	gtk_toggle_button_set_active (togglebutton, FALSE);
 }
 
+/* this function was the signal handler for 'check-resize' of main_window.
+ * I see no reason to have it further. so i unlinked it. the code is here
+ * if we need it some later time.
+ */
 void on_main_window_check_resize (GtkContainer *container,
                                             gpointer user_data)
 {
+	fprintf (stderr, _("[%s] on_main_window_check_resize should not get called. %s\n"), PROG_NAME, BUG_REPORT);
+	
 	static gboolean		itsme=FALSE;
 	
+	/* only in classic views this function may take effect */
+	if (prefs.mode == PAPER_MODE) return;
 	/* is there a nicer way to to this? */
 	if (itsme) {
 		itsme = FALSE;
@@ -1618,7 +1573,7 @@ void on_formula_entry_activate (GtkEntry *entry, gpointer user_data)
 	
 	result = flex_parser(gtk_entry_get_text(entry));
 	ui_formula_entry_state (result.error);
-	if (!result.error) display_result_set_double (result.value);
+	if (!result.error) display_result_set_double (result.value, current_status.number);
 }
 
 void on_formula_entry_changed (GtkEditable *editable, gpointer user_data)
@@ -1626,4 +1581,93 @@ void on_formula_entry_changed (GtkEditable *editable, gpointer user_data)
 	ui_formula_entry_state(FALSE);
 }
 
+void on_paper_entry_activate (GtkWidget *activated_widget, gpointer user_data)
+{
+	s_flex_parser_result 	result;
+	GtkEntry				*entry;
+	GtkTreeView				*tree_view;
+	GtkListStore			*paper_store;
+	GtkTreeIter   			iter;
+	char					*escaped_input_string, *result_string, *markup_result_string;
+	GtkTreePath* 			last_row_path;
+	
+	if (!GTK_IS_ENTRY(activated_widget))
+		entry = GTK_ENTRY(glade_xml_get_widget (view_xml, "paper_entry"));
+	else
+		entry = GTK_ENTRY(activated_widget);
+	
+	if (strcmp(gtk_entry_get_text(entry), "") == 0) return;
+	/* result.error result.value */
+	result = flex_parser(gtk_entry_get_text(entry));
+	
+	/* add to tree view */
+	tree_view = GTK_TREE_VIEW(glade_xml_get_widget (view_xml, "paper_treeview"));
+	paper_store = GTK_LIST_STORE(gtk_tree_view_get_model(tree_view));
+	gtk_list_store_append (paper_store, &iter);
+	escaped_input_string = g_markup_escape_text(gtk_entry_get_text(entry), -1);
+	gtk_list_store_set (paper_store, &iter, 0, escaped_input_string, 1, 0.0, 2, NULL, -1);
+	g_free(escaped_input_string);
+	
+	gtk_list_store_append (paper_store, &iter);
+	result_string = get_display_number_string(result.value, current_status.number);
+	markup_result_string = g_markup_printf_escaped ("<b>%s</b>", result_string);
+	
+	g_free(result_string);
+	if (result.error)
+		gtk_list_store_set (paper_store, &iter, 0, "Syntax Error", 1, 1.0, 2, "red", -1);
+	else
+		gtk_list_store_set (paper_store, &iter, 0, markup_result_string, 1, 1.0, 2, NULL, -1);
+	g_free(markup_result_string);
+	
+	/* scroll to last row */
+	last_row_path = gtk_tree_model_get_path (gtk_tree_view_get_model(tree_view), &iter);
+	gtk_tree_view_scroll_to_cell(tree_view, last_row_path, NULL, FALSE, 0., 0.);
+	
+	/* clear entry */
+	gtk_entry_set_text (entry, "");
+}
+
+gboolean paper_tree_view_selection_changed_cb (GtkWidget *widget,
+                                            GdkEventButton *event,
+                                            gpointer user_data)
+{
+	GtkTreeModel 		*model;
+	char 			*string, *stripped_string;
+	GtkWidget		*entry;
+	GtkTreeIter 		current_list_iter;
+	int			position;
+	GtkTreeSelection	*select;
+	
+	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1)) {
+		select = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+		if (gtk_tree_selection_get_selected (select, &model, &current_list_iter)) {
+			gtk_tree_model_get (model, &current_list_iter, 0, &string, -1);
+			stripped_string = g_strdup(string);
+			pango_parse_markup (string, -1, 0, NULL, &stripped_string, NULL, NULL);
+			g_free (string);
+			entry = glade_xml_get_widget (view_xml, "paper_entry");
+			position = gtk_editable_get_position (GTK_EDITABLE(entry));
+			gtk_editable_insert_text (GTK_EDITABLE (entry), stripped_string, strlen(stripped_string), &position);
+			/* set position after currently inserted text */
+			gtk_editable_set_position (GTK_EDITABLE(entry), position);
+			g_free (stripped_string);
+		}
+	}
+	/* return FALSE to let other process this signal, too */
+	return FALSE;
+}
+
+gboolean on_button_can_activate_accel (GtkWidget *widget, guint signal_id, gpointer user_data)
+{
+	if (!GTK_WIDGET_SENSITIVE(widget)) return FALSE;
+	if (strcmp("clicked", g_signal_name(signal_id)) == 0) return TRUE;
+	return FALSE;
+}
+
+gboolean on_menuitem_can_activate_accel (GtkWidget *widget, guint signal_id, gpointer user_data)
+{
+	if (!GTK_WIDGET_SENSITIVE(widget)) return FALSE;
+	if (strcmp("activate", g_signal_name(signal_id)) == 0) return TRUE;
+	return FALSE;
+}
 /* END */
