@@ -1,7 +1,7 @@
 /*
  *  ui.c - general user interface code.
  *	part of galculator
- *  	(c) 2002-2004 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2009 Simon Floery (chimaira@users.sf.net)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-
+ 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,11 +35,12 @@
 #include <glade/glade.h>
 
 GladeXML	*main_window_xml, *dispctrl_xml, *button_box_xml, *prefs_xml, 
-		*about_dialog_xml;
+		*about_dialog_xml, *view_xml;
 char		dec_point[2];
 GtkListStore	*prefs_constant_store, *prefs_user_function_store;
 
 static void set_disp_ctrl_object_data ();
+static void set_all_buttons_property (GFunc func, gpointer data);
 static void set_all_dispctrl_buttons_property (GFunc func, gpointer data);
 static void set_all_normal_buttons_property (GFunc func, gpointer data);
 static void set_table_child_callback (gpointer data, gpointer user_data);
@@ -262,8 +263,9 @@ static void ui_pack_from_xml (GtkWidget *box,
 				gboolean expand,
 				gboolean fill)
 {
-	GtkWidget	*child_widget, *accel_child_widget;
-	GtkAccelGroup	*accel_group;
+	GtkWidget	*child_widget=NULL, *accel_child_widget=NULL;
+	GtkAccelGroup	*accel_group=NULL;
+	GList		*accel_list=NULL;
 	
 	/* at first connect signal handlers */
 	glade_xml_signal_autoconnect (child_xml);
@@ -273,12 +275,13 @@ static void ui_pack_from_xml (GtkWidget *box,
 	 * to get working accelerators.
 	 */
 	accel_child_widget = glade_xml_get_widget (child_xml, accel_child_name);
-
-	accel_group = gtk_accel_group_from_accel_closure ((GClosure *) 
-		(gtk_widget_list_accel_closures (accel_child_widget))->data);
-	
-	gtk_window_add_accel_group ((GtkWindow *) gtk_widget_get_toplevel (box),
-		accel_group);
+	if (!accel_child_widget)
+		error_message ("Couldn't find widget \"%s\" in \"ui_pack_from_xml\"", accel_child_name);
+	accel_list = gtk_widget_list_accel_closures (accel_child_widget);
+	if (accel_list)
+		accel_group = gtk_accel_group_from_accel_closure ((GClosure *) accel_list->data);
+	if (accel_group)
+		gtk_window_add_accel_group ((GtkWindow *) gtk_widget_get_toplevel (box), accel_group);
 
 	gtk_box_pack_start ((GtkBox *) box, child_widget, expand, fill, 0);
 	gtk_box_reorder_child ((GtkBox *) box, child_widget, index);
@@ -313,33 +316,35 @@ void ui_main_window_set_dispctrl (int location)
 	if (dispctrl_xml) {
 		table_dispctrl = glade_xml_get_widget (dispctrl_xml, "table_dispctrl");
 		if (table_dispctrl) gtk_widget_destroy (table_dispctrl); 
-		g_free (dispctrl_xml);
+		g_object_unref (G_OBJECT(dispctrl_xml));
+		dispctrl_xml = NULL;
 	}	
 	/* now create the new one at location */
+
 	switch(location) {
 		case DISPCTRL_BOTTOM:
-			box = glade_xml_get_widget (main_window_xml, "window_vbox");
+			box = glade_xml_get_widget (view_xml, "display_vbox");
 			dispctrl_xml = glade_file_open (DISPCTRL_BOTTOM_GLADE_FILE, 
 				"table_dispctrl", TRUE);
-			ui_pack_from_xml (box, 2, dispctrl_xml, "table_dispctrl", 
+			ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl", 
 				"button_clr", TRUE, TRUE);
 			break;
 		case DISPCTRL_RIGHT:
-			box = glade_xml_get_widget (main_window_xml, "display_hbox");
+			box = glade_xml_get_widget (view_xml, "display_hbox");
 			dispctrl_xml = glade_file_open (DISPCTRL_RIGHT_GLADE_FILE, 
 				"table_dispctrl", TRUE);
 			ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl",
 				"button_clr", FALSE, FALSE);
 			break;
 		case DISPCTRL_RIGHTV:
-			box = glade_xml_get_widget (main_window_xml, "display_hbox");
+			box = glade_xml_get_widget (view_xml, "display_hbox");
 			dispctrl_xml = glade_file_open (DISPCTRL_RIGHTV_GLADE_FILE, 
 				"table_dispctrl", TRUE);
 			ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl",
 				"button_clr", FALSE, FALSE);
 			break;
 		default:
-			error_message ("Unknown mode in \"ui_main_window_set_dispctrl\"");
+			error_message ("Unknown location %i in \"ui_main_window_set_dispctrl\"", location);
 	}
 	set_disp_ctrl_object_data ();
 
@@ -348,6 +353,7 @@ void ui_main_window_set_dispctrl (int location)
 	signal_cb.detailed_signal = g_strdup ("button_press_event");
 	signal_cb.callback = (GCallback) on_button_press_event;
 	set_all_dispctrl_buttons_property (set_table_child_callback, (gpointer) &signal_cb);
+	set_all_dispctrl_buttons_tip();
 }
 
 /* ui_main_window_buttons_destroy. removes the scientific resp basic mode 
@@ -357,13 +363,10 @@ void ui_main_window_set_dispctrl (int location)
 void ui_main_window_buttons_destroy ()
 {
 	GtkWidget	*box;
-	GList		*children;
 	
-	box = glade_xml_get_widget (main_window_xml, "window_vbox");
-	children = gtk_container_get_children ((GtkContainer *)box);
-	children = g_list_last (children);
-	if (children->data != NULL) gtk_widget_destroy (children->data);
-	g_list_free (children);
+	if (!button_box_xml) return;
+	box = glade_xml_get_widget (button_box_xml, "button_box");
+	if (box) gtk_widget_destroy (box);
 }
 
 /* ui_main_window_buttons_create. fills main_window with calculator's buttons,
@@ -376,19 +379,28 @@ void ui_main_window_buttons_create (int mode)
 	struct lconv 	*locale_settings;
 	s_signal_cb	signal_cb;
 	
-	if (mode == BASIC_MODE) {
+	switch (mode) {
+	case BASIC_MODE:
+		if (button_box_xml) g_object_unref (G_OBJECT(button_box_xml));
 		button_box_xml = glade_file_open (BASIC_GLADE_FILE, "button_box", TRUE);
-		box = glade_xml_get_widget (main_window_xml, "window_vbox");
-		ui_pack_from_xml (box, 4, button_box_xml, "button_box", 
+		box = glade_xml_get_widget (view_xml, "classic_view_vbox");
+		ui_pack_from_xml (box, 2, button_box_xml, "button_box", 
 			"button_1", TRUE, TRUE);
 		set_basic_object_data (button_box_xml);
-	} else if (mode == SCIENTIFIC_MODE) {
+		break;
+	case SCIENTIFIC_MODE:
+		if (button_box_xml) g_object_unref (G_OBJECT(button_box_xml));
 		button_box_xml = glade_file_open (SCIENTIFIC_GLADE_FILE, "button_box", TRUE);
-		box = glade_xml_get_widget (main_window_xml, "window_vbox");
-		ui_pack_from_xml (box, 4, button_box_xml, "button_box", 
+		box = glade_xml_get_widget (view_xml, "classic_view_vbox");
+		ui_pack_from_xml (box, 2, button_box_xml, "button_box", 
 			"button_1", TRUE, TRUE);
 		set_scientific_object_data (button_box_xml);
-	} else error_message ("Unknown mode in \"ui_main_window_buttons_create\"");
+		break;
+	case PAPER_MODE:
+		return;
+	default:
+		error_message ("Unknown mode in \"ui_main_window_buttons_create\"");
+	}
 
 	signal_cb.detailed_signal = g_strdup ("button_press_event");
 	signal_cb.callback = (GCallback) on_button_press_event;
@@ -410,6 +422,11 @@ is not supported: >%s<\nYou might face problems when using %s! %s\n)"),
 	gtk_widget_set_sensitive (button, memory.len > 0);
 	button = glade_xml_get_widget (button_box_xml, "button_Mplus");
 	gtk_widget_set_sensitive (button, memory.len > 0);
+	
+	/* apply button specific prefs */
+	set_all_normal_buttons_size (prefs.button_width, prefs.button_height);
+	set_all_normal_buttons_font (prefs.custom_button_font ? prefs.button_font : "");
+	set_all_normal_buttons_tip ();
 }
 
 /* set_table_child_callback. Function argument for set_all_*_buttons_property.
@@ -464,6 +481,53 @@ static void set_table_child_font (gpointer data, gpointer user_data)
 	/* else do nothing */
 }
 
+gboolean set_table_child_tip_accel_finder (GtkAccelKey *key, GClosure *closure, gpointer data)
+{
+	gpointer	*d;
+	GClosure	*c;
+	GtkTooltipsData *tip_data;
+	gchar		*tt, *al;
+	
+	d = (gpointer *) data;
+	c = (GClosure *) d[0];
+	tip_data = (GtkTooltipsData *) d[1];
+	if (closure == c) {
+		tt = tip_data->tip_text;
+		al = gtk_accelerator_get_label(key->accel_key, key->accel_mods);
+		tip_data->tip_text = g_strdup_printf("%s    %s", tt, al);
+		g_free(tt);
+		g_free(al);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
+static void set_table_child_tip_accel (gpointer data, gpointer user_data)
+{
+	GtkTableChild	*tc;
+	GtkWidget	*button;
+	GList 		*closure_list;
+	GtkAccelGroup	*accel_group;
+	GtkTooltipsData *tip_data;
+	gpointer	d[2];
+	
+	tc = data;
+	button = tc->widget;
+	/* get all accelerators (== closures) connected to this button */
+	closure_list = gtk_widget_list_accel_closures (button);
+	if (!closure_list) return;
+	if (!closure_list->data) return;
+	/* we head for first closure */
+	accel_group = gtk_accel_group_from_accel_closure(closure_list->data);
+	if (!accel_group) return;
+	tip_data = gtk_tooltips_data_get(button);
+	if (!tip_data) return;
+	d[0] = (gpointer) closure_list->data;
+	d[1] = (gpointer) tip_data;
+	gtk_accel_group_find(accel_group, (GtkAccelGroupFindFunc)set_table_child_tip_accel_finder, d);
+}
+
 /* set_all_dispctrl_buttons_property. calls func with argument data for 
  * every button of the display control section.
  */
@@ -472,10 +536,11 @@ static void set_all_dispctrl_buttons_property (GFunc func, gpointer data)
 {
 	GtkTable	*table;
 
+	if (!dispctrl_xml) return;
 	/* at first the display control table. always there; somehow */
 	table = (GtkTable *) glade_xml_get_widget (dispctrl_xml, 
 		"table_dispctrl");
-	
+	if (!table) return;
 	/* dispctrl_right has an extra table for cosmetic reasons. */
 	if (GTK_IS_TABLE (((GtkTableChild *)table->children->data)->widget))
 		table = (GtkTable *) ((GtkTableChild *)table->children->data)->widget;
@@ -491,12 +556,13 @@ static void set_all_normal_buttons_property (GFunc func, gpointer data)
 	GtkTable	*table;
 	
 	/* now depending on mode the remaining buttons */
-	if (prefs.mode == BASIC_MODE) {
+	switch (prefs.mode) {
+	case BASIC_MODE:
 		table = (GtkTable *) glade_xml_get_widget (button_box_xml, 
 			"table_buttons");
 		g_list_foreach (table->children, func, data);
-	
-	} else if (prefs.mode == SCIENTIFIC_MODE) {
+		break;
+	case SCIENTIFIC_MODE:
 		table = (GtkTable *) glade_xml_get_widget (button_box_xml, 
 			"table_standard_buttons");
 		g_list_foreach (table->children, func, data);
@@ -506,8 +572,13 @@ static void set_all_normal_buttons_property (GFunc func, gpointer data)
 		table = (GtkTable *) glade_xml_get_widget (button_box_xml, 
 			"table_func_buttons");
 		g_list_foreach (table->children, func, data);
+		break;
+	case PAPER_MODE:
+		/* do nothing - no buttons */
+		break;
+	default:
+		error_message ("Unknown mode %i in \"set_all_normal_buttons_property\"", prefs.mode);
 	}
-	else error_message ("Unknown mode in \"set_all_buttons_property\"");
 }
 
 /* set_all_buttons_property. calls func with argument data for every button.
@@ -528,7 +599,45 @@ void set_all_buttons_size (int width, int height)
 	
 	size.width = width;
 	size.height = height;
+		
 	set_all_buttons_property (set_table_child_size, (gpointer) &size);
+		
+	gtk_window_resize ((GtkWindow *) gtk_widget_get_toplevel(
+			(GtkWidget *) glade_xml_get_widget (main_window_xml, "main_window")), 1, 1);
+}
+
+void set_all_normal_buttons_tip ()
+{
+	set_all_normal_buttons_property (set_table_child_tip_accel, NULL);
+}
+
+void set_all_dispctrl_buttons_tip ()
+{
+	set_all_dispctrl_buttons_property (set_table_child_tip_accel, NULL);
+}
+
+/* set_all_nomral_buttons_size. gateway for set_all_normal_buttons_property.
+ */
+
+void set_all_normal_buttons_size (int width, int height)
+{
+	GtkRequisition	size;
+	
+	size.width = width;
+	size.height = height;
+	set_all_normal_buttons_property (set_table_child_size, (gpointer) &size);
+}
+
+/* set_all_dispctrl_buttons_size. gateway for set_all_dispctrl_buttons_property.
+ */
+
+void set_all_dispctrl_buttons_size (int width, int height)
+{
+	GtkRequisition	size;
+	
+	size.width = width;
+	size.height = height;
+	set_all_dispctrl_buttons_property (set_table_child_size, (gpointer) &size);
 }
 
 /* set_all_buttons_font. gateway for set_all_buttons_property.
@@ -542,6 +651,28 @@ void set_all_buttons_font (char *font_string)
 	set_all_buttons_property (set_table_child_font, pango_font);
 }
 
+/* set_all_normal_buttons_font. gateway for set_all_buttons_property.
+ */
+
+void set_all_normal_buttons_font (char *font_string)
+{
+	PangoFontDescription	*pango_font;
+
+	pango_font = pango_font_description_from_string (font_string);
+	set_all_normal_buttons_property (set_table_child_font, pango_font);
+}
+
+/* set_all_dispctrl_buttons_font. gateway for set_all_buttons_property.
+ */
+
+void set_all_dispctrl_buttons_font (char *font_string)
+{
+	PangoFontDescription	*pango_font;
+
+	pango_font = pango_font_description_from_string (font_string);
+	set_all_dispctrl_buttons_property (set_table_child_font, pango_font);
+}
+
 gboolean button_deactivation (gpointer data)
 {
 	GtkToggleButton 	*b;
@@ -588,6 +719,9 @@ void update_dispctrl()
 	else ui_main_window_set_dispctrl (DISPCTRL_RIGHT);
 	set_widget_visibility (dispctrl_xml, "table_dispctrl", 
 		prefs.vis_dispctrl);
+	set_all_dispctrl_buttons_size (prefs.button_width, prefs.button_height);
+	set_all_dispctrl_buttons_font (prefs.custom_button_font ? prefs.button_font : "");
+
 }
 
 /*
@@ -599,56 +733,14 @@ void set_widget_visibility (GladeXML *xml, char *widget_name, gboolean visible)
 	GtkWidget	*widget;
 	
 	widget = glade_xml_get_widget (xml, widget_name);
+	if (!widget) {
+		error_message ("Couldn't find widget \"%s\" in \"set_widget_visibility\"", widget_name);
+		return;
+	}
 	if (visible) gtk_widget_show_all (widget);
 	else gtk_widget_hide_all (widget);
 }
 
-GtkWidget *ui_font_dialog_create (char *title, GtkButton *button)
-{
-	GladeXML	*font_xml;
-	GtkWidget	*font_dialog;
-	
-	font_xml = glade_file_open (FONT_GLADE_FILE, "font_dialog", FALSE);
-	glade_xml_signal_autoconnect(font_xml);
-	font_dialog = glade_xml_get_widget (font_xml, "font_dialog");
-	
-	gtk_window_set_title ((GtkWindow *) font_dialog, title);
-	gtk_font_selection_dialog_set_font_name ((GtkFontSelectionDialog *) font_dialog, \
-		gtk_button_get_label (button));
-	gtk_widget_hide (((GtkFontSelectionDialog *)font_dialog)->apply_button);
-	gtk_widget_show (font_dialog);
-
-	return font_dialog;
-}
-
-GtkWidget *ui_color_dialog_create (char *title, GtkButton *button)
-{
-	GladeXML	*color_xml;
-	GtkWidget	*color_dialog, *da;
-	GdkColor	color;
-	GtkRcStyle	*style;
-	GtkBox		*box;
-	GtkBin		*vp;
-	
-	box = (GtkBox *)gtk_bin_get_child ((GtkBin *) button);
-	vp = (GtkBin *) (((GtkBoxChild *)g_list_nth_data (box->children, 1))->widget);
-	da = gtk_bin_get_child(vp);
-	
-	style = gtk_widget_get_modifier_style (da);
-	color = style->fg[GTK_STATE_NORMAL];
-	
-	color_xml = glade_file_open (COLOR_GLADE_FILE, "color_dialog", FALSE);
-	glade_xml_signal_autoconnect(color_xml);
-	color_dialog = glade_xml_get_widget (color_xml, "color_dialog");
-	
-	gtk_window_set_title ((GtkWindow *) color_dialog, title);
-	gtk_color_selection_set_current_color ((GtkColorSelection *)((GtkColorSelectionDialog *)color_dialog)->colorsel, \
-		&color);
-	gtk_widget_hide (((GtkColorSelectionDialog *)color_dialog)->help_button);
-	gtk_widget_show (color_dialog);
-	return color_dialog;
-}
-
 /* menu code - e.g. used for the constant popup menu */
 
 void position_menu (GtkMenu *menu, 
@@ -801,7 +893,6 @@ GtkWidget *ui_pref_dialog_create ()
 	GtkSizeGroup		*sgroup;
 	s_prefs_entry		*prefs_list;
 	
-	
 	prefs_xml = glade_file_open (PREFS_GLADE_FILE, "prefs_dialog", FALSE);
 	glade_xml_signal_autoconnect(prefs_xml);
 	
@@ -813,13 +904,14 @@ GtkWidget *ui_pref_dialog_create ()
 	prefs_list = config_file_get_prefs_list();
 	while (prefs_list[counter].key != NULL) {
 		if (prefs_list[counter].set_handler != NULL) {
-			prefs_list[counter].set_handler (prefs_xml,
+			prefs_list[counter].set_handler (
+				prefs_xml,
 				prefs_list[counter].widget_name,
 				prefs_list[counter].variable);
 		}
 		counter++;
 	}
-	
+
 	w = glade_xml_get_widget (prefs_xml, "prefs_button_font_label");
 	gtk_widget_set_sensitive (w, prefs.custom_button_font);
 	
@@ -828,7 +920,7 @@ GtkWidget *ui_pref_dialog_create ()
 	
 	w = glade_xml_get_widget (prefs_xml, "prefs_bin_length");
 	gtk_widget_set_sensitive (w, prefs.bin_fixed);
-
+	
 	/* make user defined constants list. */
 	
 	prefs_constant_store = gtk_list_store_new (NR_CONST_COLUMNS, 
@@ -923,7 +1015,7 @@ GtkWidget *ui_pref_dialog_create ()
 		glade_xml_get_widget (prefs_xml, "prefs_func_delete_button"));
 	gtk_size_group_add_widget (sgroup,
 		glade_xml_get_widget (prefs_xml, "prefs_func_clear_button"));
-
+	
 	gtk_widget_show (prefs_dialog);
 	return prefs_dialog;
 }
@@ -940,7 +1032,7 @@ void ui_formula_entry_activate ()
 {
 	GtkWidget	*formula_entry;
 	
-	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	formula_entry = glade_xml_get_widget (view_xml, "formula_entry");
 	gtk_widget_activate(formula_entry);
 }
 
@@ -949,7 +1041,7 @@ void ui_formula_entry_set (G_CONST_RETURN gchar *text)
 	GtkWidget	*formula_entry;
 
 	if (text == NULL) return;
-	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	formula_entry = glade_xml_get_widget (view_xml, "formula_entry");
 	gtk_entry_set_text ((GtkEntry *) formula_entry, text);
 }
 
@@ -959,7 +1051,7 @@ void ui_formula_entry_insert (G_CONST_RETURN gchar *text)
 	int		position;
 	
 	if (text == NULL) return;
-	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	formula_entry = glade_xml_get_widget (view_xml, "formula_entry");
 	position = gtk_editable_get_position ((GtkEditable *) formula_entry);
 	gtk_editable_insert_text ((GtkEditable *) formula_entry, text, -1,
                                              &position);
@@ -970,7 +1062,7 @@ void ui_formula_entry_backspace ()
 {
 	GtkWidget	*formula_entry;
 	
-	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	formula_entry = glade_xml_get_widget (view_xml, "formula_entry");
 	gtk_editable_delete_text ((GtkEditable *) formula_entry, 
 		strlen(gtk_entry_get_text((GtkEntry *) formula_entry)) - 1, -1);
 }
@@ -984,7 +1076,7 @@ void ui_formula_entry_state (gboolean error)
 	GtkWidget		*formula_entry;
 	GdkColor		*color=NULL;
 	
-	formula_entry = glade_xml_get_widget (main_window_xml, "formula_entry");
+	formula_entry = glade_xml_get_widget (view_xml, "formula_entry");
 	if (error) {
 		color = (GdkColor *) malloc(sizeof(GdkColor));
 		gdk_color_parse ("red", color);
@@ -1030,3 +1122,79 @@ void ui_relax_fmod_buttons ()
 	tbutton = glade_xml_get_widget (button_box_xml, "button_hyp");
 	gtk_toggle_button_set_active ((GtkToggleButton *) tbutton, FALSE);
 }
+
+void ui_classic_view_create()
+{
+	GtkWidget	*classic_view_vbox, *box;
+	
+	/* at first, check if there is already a classic view */
+	if (view_xml) {
+		classic_view_vbox = glade_xml_get_widget (view_xml, "classic_view_vbox");
+		if (classic_view_vbox) return;
+	}
+	
+	/* if not, build one */
+	view_xml = glade_file_open (CLASSIC_VIEW_GLADE_FILE, "classic_view_vbox", TRUE);
+	box = glade_xml_get_widget (main_window_xml, "window_vbox");
+	ui_pack_from_xml (box, 1, view_xml, "classic_view_vbox", "textview", TRUE, TRUE);
+	
+	display_init ();
+	
+	remember_display_values ();
+}
+
+void ui_classic_view_destroy()
+{
+	GtkWidget	*classic_view_vbox;
+	
+	if (!view_xml) return;
+	classic_view_vbox = glade_xml_get_widget (view_xml, "classic_view_vbox");
+	if (classic_view_vbox) gtk_widget_destroy (classic_view_vbox);
+}
+
+void ui_paper_view_create()
+{
+	GtkWidget		*paper_view_vbox, *box, *tree_view;
+	GtkCellRenderer 	*renderer;
+	GtkTreeViewColumn 	*column;
+	GtkListStore		*paper_store;
+	GtkTreeSelection	*select;
+
+	/* at first, check if there is already a ng view */
+	if (view_xml) {
+		paper_view_vbox = glade_xml_get_widget (view_xml, "paper_view_vbox");
+		if (paper_view_vbox) return;
+	}
+	
+	/* if not, build one */
+	view_xml = glade_file_open (PAPER_VIEW_GLADE_FILE, "paper_view_vbox", TRUE);
+	box = glade_xml_get_widget (main_window_xml, "window_vbox");
+	ui_pack_from_xml (box, 1, view_xml, "paper_view_vbox", "paper_treeview", TRUE, TRUE);
+	
+	/* markup / xalign / foreground */
+	paper_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_STRING);
+	tree_view = glade_xml_get_widget (view_xml, "paper_treeview");
+	gtk_tree_view_set_model ((GtkTreeView *) tree_view, GTK_TREE_MODEL (paper_store));
+	
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes ("Result Display", renderer, "markup", 0, "xalign", 1, "foreground", 2, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
+	
+	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+	g_signal_connect (G_OBJECT (tree_view), "button-press-event",
+                  G_CALLBACK (paper_tree_view_selection_changed_cb),
+                  NULL);
+                  
+	gtk_widget_grab_focus(glade_xml_get_widget (view_xml, "paper_entry"));
+
+}
+
+void ui_paper_view_destroy()
+{
+	GtkWidget	*paper_view_vbox;
+	
+	if (!view_xml) return;
+	paper_view_vbox = glade_xml_get_widget (view_xml, "paper_view_vbox");
+	if (paper_view_vbox) gtk_widget_destroy (paper_view_vbox);
+}
